@@ -4,7 +4,7 @@
  *	This module implements a graph widget for the
  *	Tk toolkit.
  *
- * Copyright 1991-1993 by AT&T Bell Laboratories.
+ * Copyright 1991-1994 by AT&T Bell Laboratories.
  * Permission to use, copy, modify, and distribute this software
  * and its documentation for any purpose and without fee is hereby
  * granted, provided that the above copyright notice appear in all
@@ -28,6 +28,8 @@
 #include "bltGraph.h"
 #include <X11/Xutil.h>
 
+extern int strcasecmp _ANSI_ARGS_((CONST char *s1, CONST char *s2));
+
 /* ----------------------------------------------------------------------
  * Custom option parse and print procedures
  * ----------------------------------------------------------------------
@@ -127,6 +129,108 @@ Blt_PrintCoords(clientData, tkwin, widgRec, offset, freeProcPtr)
     }
     return (result);
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * ParseAxisFlags --
+ *
+ *	Convert the string indicating which axis flags to use.
+ *
+ * Results:
+ *	The return value is a standard Tcl result.  The axis flags are
+ *	written into the widget record.
+ *
+ *----------------------------------------------------------------------
+ */
+/*ARGSUSED*/
+int
+Blt_ParseAxisFlags(clientData, interp, tkwin, value, widgRec, offset)
+    ClientData clientData;	/* not used */
+    Tcl_Interp *interp;		/* Interpreter to send results back to */
+    Tk_Window tkwin;		/* not used */
+    char *value;		/* Axis type */
+    char *widgRec;		/* Widget record */
+    int offset;			/* offset of AxisType field */
+{
+    int *flagsPtr = (int *)(widgRec + offset);
+    int mask = (int)clientData;
+    int flags;
+
+    flags = 0;
+    if (mask == AXIS_X_MASK) {
+	if (strcmp(value, "x2") == 0) {
+	    flags = AXIS_X2_FLAG;
+	} else if (strcmp(value, "x1") == 0) {
+	    flags = AXIS_X1_FLAG;
+	} else if (strcmp(value, "x") == 0) {
+	    flags = AXIS_X1_FLAG;
+	} else if (strcmp(value, "both") == 0) {
+	    flags = (AXIS_X1_FLAG | AXIS_X2_FLAG);
+	} else {
+	    Tcl_AppendResult(interp, "bad x-axis type \"", value,
+		"\": should be x or x2", (char *)NULL);
+	    return TCL_ERROR;
+	}
+    } else {
+	if (strcmp(value, "y2") == 0) {
+	    flags = AXIS_Y2_FLAG;
+	} else if (strcmp(value, "y1") == 0) {
+	    flags = AXIS_Y1_FLAG;
+	} else if (strcmp(value, "y") == 0) {
+	    flags = AXIS_Y1_FLAG;
+	} else if (strcmp(value, "both") == 0) {
+	    flags = (AXIS_Y1_FLAG | AXIS_Y2_FLAG);
+	} else {
+	    Tcl_AppendResult(interp, "bad y-axis type \"", value,
+		"\": should be y or y2", (char *)NULL);
+	    return TCL_ERROR;
+	}
+    }
+    *flagsPtr &= ~mask;
+    *flagsPtr |= flags;
+    return TCL_OK;
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Blt_PrintAxisFlags --
+ *
+ *	Convert the window coordinates into a string.
+ *
+ * Results:
+ *	The string representing the coordinate position is returned.
+ *
+ *----------------------------------------------------------------------
+ */
+/*ARGSUSED*/
+char *
+Blt_PrintAxisFlags(clientData, tkwin, widgRec, offset, freeProcPtr)
+    ClientData clientData;	/* not used */
+    Tk_Window tkwin;		/* not used */
+    char *widgRec;		/* Widget record */
+    int offset;			/* offset of AxisType field */
+    Tcl_FreeProc **freeProcPtr;	/* not used */
+{
+    char *result;
+    int flags = *(int *)(widgRec + offset);
+    int mask = (int)clientData;
+
+    flags &= mask;
+    if ((flags == AXIS_X_MASK) || (flags == AXIS_Y_MASK)) {
+	result = "both";
+    } else if (flags & AXIS_X1_FLAG) {
+	result = "x";
+    } else if (flags & AXIS_X2_FLAG) {
+	result = "x2";
+    } else if (flags & AXIS_Y1_FLAG) {
+	result = "y";
+    } else if (flags & AXIS_Y2_FLAG) {
+	result = "y2";
+    }
+    return (result);
+}
 
 /*-----------------------------------------------------------------
  * X-related drawing routines
@@ -184,8 +288,7 @@ XPoint
 Blt_TranslateTextCoords(fontPtr, text, x, y, anchor)
     XFontStruct *fontPtr;	/* Font information */
     char *text;			/* Text string */
-    int x;			/* X position of anchor */
-    int y;			/* Y position of anchor */
+    int x, y;			/* Position of anchor */
     Tk_Anchor anchor;		/* Direction of the anchor */
 {
     int width;
@@ -200,39 +303,39 @@ Blt_TranslateTextCoords(fontPtr, text, x, y, anchor)
 
     newPt.x = x, newPt.y = y;
     switch (anchor) {
-	case TK_ANCHOR_NW:
-	    newPt.y += fontPtr->ascent;
-	    break;
-	case TK_ANCHOR_W:
-	    newPt.y += (fontPtr->ascent - fontPtr->descent) / 2;
-	    break;
-	case TK_ANCHOR_SW:
-	    newPt.y -= fontPtr->descent;
-	    break;
-	case TK_ANCHOR_NE:
-	    newPt.x -= width;
-	    newPt.y += fontPtr->ascent;
-	    break;
-	case TK_ANCHOR_E:
-	    newPt.x -= width;
-	    newPt.y += (fontPtr->ascent - fontPtr->descent) / 2;
-	    break;
-	case TK_ANCHOR_SE:
-	    newPt.x -= width;
-	    newPt.y -= fontPtr->descent;
-	    break;
-	case TK_ANCHOR_N:
-	    newPt.y += fontPtr->ascent;
-	    newPt.x -= width / 2;
-	    break;
-	case TK_ANCHOR_S:
-	    newPt.y -= fontPtr->descent;
-	    newPt.x -= width / 2;
-	    break;
-	case TK_ANCHOR_CENTER:
-	    newPt.x -= width / 2;
-	    newPt.y += (fontPtr->ascent - fontPtr->descent) / 2;
-	    break;
+    case TK_ANCHOR_NW:
+	newPt.y += fontPtr->ascent;
+	break;
+    case TK_ANCHOR_W:
+	newPt.y += (fontPtr->ascent - fontPtr->descent) / 2;
+	break;
+    case TK_ANCHOR_SW:
+	newPt.y -= fontPtr->descent;
+	break;
+    case TK_ANCHOR_NE:
+	newPt.x -= width;
+	newPt.y += fontPtr->ascent;
+	break;
+    case TK_ANCHOR_E:
+	newPt.x -= width;
+	newPt.y += (fontPtr->ascent - fontPtr->descent) / 2;
+	break;
+    case TK_ANCHOR_SE:
+	newPt.x -= width;
+	newPt.y -= fontPtr->descent;
+	break;
+    case TK_ANCHOR_N:
+	newPt.y += fontPtr->ascent;
+	newPt.x -= width / 2;
+	break;
+    case TK_ANCHOR_S:
+	newPt.y -= fontPtr->descent;
+	newPt.x -= width / 2;
+	break;
+    case TK_ANCHOR_CENTER:
+	newPt.x -= width / 2;
+	newPt.y += (fontPtr->ascent - fontPtr->descent) / 2;
+	break;
     }
     return (newPt);
 }
@@ -261,11 +364,10 @@ Blt_TranslateTextCoords(fontPtr, text, x, y, anchor)
  * -----------------------------------------------------------------
  */
 void
-Blt_GetBoundingBox(width, height, theta, rotWidthPtr, rotHeightPtr, pointArr)
-    int width, height;		/* Unrotated region */
+Blt_GetBoundingBox(width, height, theta, rotWPtr, rotHPtr, pointArr)
+    unsigned int width, height;	/* Unrotated region */
     double theta;		/* Rotation of box */
-    int *rotWidthPtr;		/* Rotated bounding box width */
-    int *rotHeightPtr;		/* Rotated bounding box height */
+    unsigned int *rotWPtr, *rotHPtr;	/* Rotated bounding box region */
     XPoint *pointArr;		/* Points of the rotated box */
 {
     register int i;
@@ -309,12 +411,12 @@ Blt_GetBoundingBox(width, height, theta, rotWidthPtr, rotHeightPtr, pointArr)
     }
 
     /*
-     * By symmetry, the width and height of the bounding box are 
+     * By symmetry, the width and height of the bounding box are
      * twice the maximum x and y coordinates.
      */
 
-    *rotWidthPtr = (int)((maxX + maxX) + 0.5);
-    *rotHeightPtr = (int)((maxY + maxY) + 0.5);
+    *rotWPtr = (int)((maxX + maxX) + 0.5);
+    *rotHPtr = (int)((maxY + maxY) + 0.5);
 }
 
 /*
@@ -343,43 +445,43 @@ Blt_GetBoundingBox(width, height, theta, rotWidthPtr, rotHeightPtr, pointArr)
 XPoint
 Blt_TranslateBoxCoords(x, y, width, height, anchor)
     int x, y;			/* Window coordinates of anchor */
-    int width, height;		/* Extents of the bounding box */
+    unsigned int width, height;	/* Extents of the bounding box */
     Tk_Anchor anchor;		/* Direction of the anchor */
 {
     XPoint newPt;
 
     newPt.x = x, newPt.y = y;
     switch (anchor) {
-	case TK_ANCHOR_NW:	/* Upper left corner */
-	    break;
-	case TK_ANCHOR_W:	/* Left center */
-	    newPt.y -= (height / 2);
-	    break;
-	case TK_ANCHOR_SW:	/* Lower left corner */
-	    newPt.y -= height;
-	    break;
-	case TK_ANCHOR_N:	/* Top center */
-	    newPt.x -= (width / 2);
-	    break;
-	case TK_ANCHOR_CENTER:	/* Centered */
-	    newPt.x -= (width / 2);
-	    newPt.y -= (height / 2);
-	    break;
-	case TK_ANCHOR_S:	/* Bottom center */
-	    newPt.x -= (width / 2);
-	    newPt.y -= height;
-	    break;
-	case TK_ANCHOR_NE:	/* Upper right corner */
-	    newPt.x -= width;
-	    break;
-	case TK_ANCHOR_E:	/* Right center */
-	    newPt.x -= width;
-	    newPt.y -= (height / 2);
-	    break;
-	case TK_ANCHOR_SE:	/* Lower right corner */
-	    newPt.x -= width;
-	    newPt.y -= height;
-	    break;
+    case TK_ANCHOR_NW:		/* Upper left corner */
+	break;
+    case TK_ANCHOR_W:		/* Left center */
+	newPt.y -= (height / 2);
+	break;
+    case TK_ANCHOR_SW:		/* Lower left corner */
+	newPt.y -= height;
+	break;
+    case TK_ANCHOR_N:		/* Top center */
+	newPt.x -= (width / 2);
+	break;
+    case TK_ANCHOR_CENTER:	/* Centered */
+	newPt.x -= (width / 2);
+	newPt.y -= (height / 2);
+	break;
+    case TK_ANCHOR_S:		/* Bottom center */
+	newPt.x -= (width / 2);
+	newPt.y -= height;
+	break;
+    case TK_ANCHOR_NE:		/* Upper right corner */
+	newPt.x -= width;
+	break;
+    case TK_ANCHOR_E:		/* Right center */
+	newPt.x -= width;
+	newPt.y -= (height / 2);
+	break;
+    case TK_ANCHOR_SE:		/* Lower right corner */
+	newPt.x -= width;
+	newPt.y -= height;
+	break;
     }
     return (newPt);
 }
@@ -399,16 +501,16 @@ Blt_TranslateBoxCoords(x, y, width, height, anchor)
  * -----------------------------------------------------------------
  */
 Pixmap
-Blt_RotateBitmap(display, draw, bitmapGC, srcBitmap, srcWidth, srcHeight, 
-	theta, rotWidthPtr, rotHeightPtr)
+Blt_RotateBitmap(display, draw, bitmapGC, srcBitmap, srcWidth, srcHeight,
+    theta, rotWPtr, rotHPtr)
     Display *display;		/* X display */
     Drawable draw;		/* Root window drawable */
     GC bitmapGC;		/* GC created from bitmap where fg=1,bg=0 */
     Pixmap srcBitmap;		/* Source bitmap to be rotated */
-    int srcWidth, srcHeight;	/* Width and height of the source bitmap */
+    unsigned int srcWidth;	/* Width and height of the source bitmap */
+    unsigned int srcHeight;
     double theta;		/* Right angle rotation to perform */
-    unsigned int *rotWidthPtr;
-    unsigned int *rotHeightPtr;
+    unsigned int *rotWPtr, *rotHPtr;
 {
     XImage *src, *dest;
     Pixmap destBitmap;
@@ -421,7 +523,7 @@ Blt_RotateBitmap(display, draw, bitmapGC, srcBitmap, srcWidth, srcHeight,
     unsigned long pixel;
     double srcX, srcY;		/* Center of source rectangle */
     double destX, destY;	/* Center of destination rectangle */
-    
+
     /*
      * Create a bitmap and image big enough to contain the rotated text
      */
@@ -436,7 +538,7 @@ Blt_RotateBitmap(display, draw, bitmapGC, srcBitmap, srcWidth, srcHeight,
 	ZPixmap);
     radians = (theta / 180.0) * M_PI;
     sinTheta = sin(radians), cosTheta = cos(radians);
-    /* 
+    /*
      * Coordinates of the centers of the source and destination rectangles
      */
     srcX = srcWidth * 0.5;
@@ -469,7 +571,7 @@ Blt_RotateBitmap(display, draw, bitmapGC, srcBitmap, srcWidth, srcHeight,
 		rotY = (transX * sinTheta) + (transY * cosTheta);
 
 		/* Translate back to the center of the source image */
-		
+
 		sx = ROUND(srcX + rotX);
 		sy = ROUND(srcY + rotY);
 
@@ -498,8 +600,8 @@ Blt_RotateBitmap(display, draw, bitmapGC, srcBitmap, srcWidth, srcHeight,
     /* Clean up temporary resources used */
 
     XDestroyImage(src), XDestroyImage(dest);
-    *rotWidthPtr = destWidth;
-    *rotHeightPtr = destHeight;
+    *rotWPtr = destWidth;
+    *rotHPtr = destHeight;
     return (destBitmap);
 }
 
@@ -521,15 +623,13 @@ Blt_RotateBitmap(display, draw, bitmapGC, srcBitmap, srcWidth, srcHeight,
  * -----------------------------------------------------------------
  */
 Pixmap
-Blt_CreateTextBitmap(display, draw, fontPtr, text, theta, bmWidthPtr,
-    bmHeightPtr)
+Blt_CreateTextBitmap(display, draw, fontPtr, text, theta, widthPtr, heightPtr)
     Display *display;
     Drawable draw;
     XFontStruct *fontPtr;	/* Font to draw bitmap */
     char *text;			/* Text string to draw */
     double theta;		/* Desired rotation of bitmap */
-    unsigned int *bmWidthPtr;	/* Width of text bitmap */
-    unsigned int *bmHeightPtr;	/* Height of text bitmap */
+    unsigned int *widthPtr, *heightPtr;	/* Height of text bitmap */
 {
     unsigned int width, height;	/* Width and height of text bounding box */
     Pixmap bitmap;
@@ -566,12 +666,12 @@ Blt_CreateTextBitmap(display, draw, fontPtr, text, theta, bmWidthPtr,
     XSetForeground(display, bitmapGC, 1);
     XDrawString(display, bitmap, bitmapGC, 0, fontPtr->ascent, text, numChar);
     if (theta == 0.0) {
-	*bmWidthPtr = width, *bmHeightPtr = height;
+	*widthPtr = width, *heightPtr = height;
     } else {
 	Pixmap rotBitmap;
 
 	rotBitmap = Blt_RotateBitmap(display, draw, bitmapGC, bitmap, width,
-	    height, theta, bmWidthPtr, bmHeightPtr);
+	    height, theta, widthPtr, heightPtr);
 	XFreePixmap(display, bitmap);
 	bitmap = rotBitmap;
     }
