@@ -4,7 +4,7 @@
  *	This module implements tags in a graph widget for
  *	the Tk toolkit.
  *
- * Copyright 1991-1993 by AT&T Bell Laboratories.
+ * Copyright 1991-1994 by AT&T Bell Laboratories.
  * Permission to use, copy, modify, and distribute this software
  * and its documentation for any purpose and without fee is hereby
  * granted, provided that the above copyright notice appear in all
@@ -29,17 +29,22 @@
 #include "bltGrTag.h"
 #include <ctype.h>
 
-#ifdef hpux
-#define BLACK		"black"
-#define WHITE		"white"
-#define BISQUE1		"#ffe4c4"
-#else
-#define BLACK		"#000000"
-#define WHITE		"#ffffff"
-#define BISQUE1		"#ffe4c4"
-#endif
+extern int Blt_ParseAxisFlags _ANSI_ARGS_((ClientData, Tcl_Interp *, Tk_Window,
+	char *, char *, int));
+extern char *Blt_PrintAxisFlags _ANSI_ARGS_((ClientData, Tk_Window, char *, int,
+	Tcl_FreeProc **));
+
+static Tk_CustomOption XAxisFlagsOption =
+{
+    Blt_ParseAxisFlags, Blt_PrintAxisFlags, (ClientData)AXIS_X_MASK
+};
+static Tk_CustomOption YAxisFlagsOption =
+{
+    Blt_ParseAxisFlags, Blt_PrintAxisFlags, (ClientData)AXIS_Y_MASK
+};
 
 #define DEF_TAG_ANCHOR		"center"
+#define DEF_TAG_AXIS		"y"
 #define DEF_TAG_BG_COLOR	WHITE
 #define DEF_TAG_BG_MONO		WHITE
 #define DEF_TAG_FG_COLOR	BLACK
@@ -60,6 +65,8 @@ typedef struct {
     unsigned int numCoords;	/* Number of points */
     Tk_ConfigSpec *configSpecs;	/* Configuration specifications */
     Tk_Uid elemId;		/* Element associated with tag */
+    int axisFlags;		/* Indicates which axes to element's map
+				 * coordinates onto */
 
     DisplayTagProc *displayProc;
     DestroyTagProc *destroyProc;
@@ -112,6 +119,8 @@ static Tk_ConfigSpec textConfigSpecs[] =
 	TK_CONFIG_MONO_ONLY | TK_CONFIG_NULL_OK},
     {TK_CONFIG_SYNONYM, "-bg", "textTagBackground", "TagBackground",
 	(char *)NULL, 0, 0},
+    {TK_CONFIG_UID, "-element", "textTagElement", "TagElement",
+	(char *)NULL, Tk_Offset(TextTag, elemId), TK_CONFIG_NULL_OK},
     {TK_CONFIG_SYNONYM, "-fg", "textTagForeground", "TagForeground",
 	(char *)NULL, 0, 0},
     {TK_CONFIG_FONT, "-font", "textTagFont", "TagFont",
@@ -121,8 +130,12 @@ static Tk_ConfigSpec textConfigSpecs[] =
 	TK_CONFIG_COLOR_ONLY},
     {TK_CONFIG_COLOR, "-foreground", "textTagForeground", "TagForeground",
 	DEF_TAG_FG_MONO, Tk_Offset(TextTag, fgColorPtr), TK_CONFIG_MONO_ONLY},
-    {TK_CONFIG_UID, "-element", "textTagElement", "TagElement",
-	(char *)NULL, Tk_Offset(TextTag, elemId), TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-mapx", "textTagMapX", "TagMapX",
+	DEF_TAG_AXIS, Tk_Offset(TextTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &XAxisFlagsOption},
+    {TK_CONFIG_CUSTOM, "-mapy", "textTagMapY", "TagMapY",
+	DEF_TAG_AXIS, Tk_Offset(TextTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &YAxisFlagsOption},
     {TK_CONFIG_DOUBLE, "-rotate", "textTagRotate", "TagRotate",
 	(char *)NULL, Tk_Offset(TextTag, theta), 0},
     {TK_CONFIG_STRING, "-text", "textTagText", "TagText",
@@ -142,6 +155,8 @@ typedef struct {
     unsigned int numCoords;	/* Number of points */
     Tk_ConfigSpec *configSpecs;	/* Configuration specifications */
     Tk_Uid elemId;		/* Element associated with tag */
+    int axisFlags;		/* Indicates which axis to element's map
+				 * y-coordinates onto */
 
     DisplayTagProc *displayProc;
     DestroyTagProc *destroyProc;
@@ -165,6 +180,12 @@ typedef struct {
 
 static Tk_ConfigSpec windowConfigSpecs[] =
 {
+    {TK_CONFIG_CUSTOM, "-mapx", "winTagMapX", "TagMapX",
+	DEF_TAG_AXIS, Tk_Offset(WindowTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &XAxisFlagsOption},
+    {TK_CONFIG_CUSTOM, "-mapy", "winTagMapY", "TagMapY",
+	DEF_TAG_AXIS, Tk_Offset(WindowTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &YAxisFlagsOption},
     {TK_CONFIG_ANCHOR, "-anchor", "winTagAnchor", "TagAnchor",
 	DEF_TAG_ANCHOR, Tk_Offset(WindowTag, anchor), 0},
     {TK_CONFIG_UID, "-element", "winTagElement", "TagElement",
@@ -190,6 +211,8 @@ typedef struct {
     unsigned int numCoords;	/* Number of points */
     Tk_ConfigSpec *configSpecs;	/* Configuration specifications */
     Tk_Uid elemId;		/* Element associated with tag */
+    int axisFlags;		/* Indicates which axes to element's map
+				 * coordinates onto */
 
     DisplayTagProc *displayProc;
     DestroyTagProc *destroyProc;
@@ -229,6 +252,12 @@ static Tk_ConfigSpec bitmapConfigSpecs[] =
 	TK_CONFIG_MONO_ONLY | TK_CONFIG_NULL_OK},
     {TK_CONFIG_SYNONYM, "-bg", "bmTagBackground", (char *)NULL, (char *)NULL,
 	0, 0},
+    {TK_CONFIG_CUSTOM, "-mapx", "bmTagMapX", "TagMapX",
+	DEF_TAG_AXIS, Tk_Offset(BitmapTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &XAxisFlagsOption},
+    {TK_CONFIG_CUSTOM, "-mapy", "bmTagMapY", "TagMapY",
+	DEF_TAG_AXIS, Tk_Offset(BitmapTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &YAxisFlagsOption},
     {TK_CONFIG_BITMAP, "-pattern", "bmTagPattern", "TagPattern",
 	(char *)NULL, Tk_Offset(BitmapTag, bitmap), TK_CONFIG_NULL_OK},
     {TK_CONFIG_SYNONYM, "-fg", "bmTagForeground", (char *)NULL, (char *)NULL,
@@ -258,6 +287,8 @@ typedef struct {
     unsigned int numCoords;	/* Number of points */
     Tk_ConfigSpec *configSpecs;	/* Configuration specifications */
     Tk_Uid elemId;		/* Element associated with tag */
+    int axisFlags;		/* Indicates which axes to element's map
+				 * coordinates onto */
 
     DisplayTagProc *displayProc;
     DestroyTagProc *destroyProc;
@@ -323,6 +354,12 @@ static Tk_ConfigSpec lineConfigSpecs[] =
 	TK_CONFIG_COLOR_ONLY},
     {TK_CONFIG_COLOR, "-foreground", "lineTagForeground", "TagForeground",
 	DEF_TAG_FG_MONO, Tk_Offset(LineTag, fgColorPtr), TK_CONFIG_MONO_ONLY},
+    {TK_CONFIG_CUSTOM, "-mapx", "lineTagMapX", "TagMapX",
+	DEF_TAG_AXIS, Tk_Offset(LineTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &XAxisFlagsOption},
+    {TK_CONFIG_CUSTOM, "-mapy", "lineTagMapY", "TagMapY",
+	DEF_TAG_AXIS, Tk_Offset(LineTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &YAxisFlagsOption},
     {TK_CONFIG_PIXELS, "-width", "lineTagLineWidth", "TagLineWidth",
 	(char *)NULL, Tk_Offset(LineTag, lineWidth), 0},
     {TK_CONFIG_PIXELS, "-xoffset", "lineTagXOffset", "TagXOffset",
@@ -340,6 +377,8 @@ typedef struct {
     unsigned int numCoords;	/* Number of points */
     Tk_ConfigSpec *configSpecs;	/* Configuration specifications */
     Tk_Uid elemId;		/* Element associated with tag */
+    int axisFlags;		/* Indicates which axis to element's map
+				 * y-coordinates onto */
 
     DisplayTagProc *displayProc;
     DestroyTagProc *destroyProc;
@@ -383,6 +422,12 @@ static Tk_ConfigSpec polygonConfigSpecs[] =
     {TK_CONFIG_COLOR, "-foreground", "polyTagForeground", "TagForeground",
 	DEF_TAG_FG_MONO, Tk_Offset(PolygonTag, fgColorPtr),
 	TK_CONFIG_MONO_ONLY},
+    {TK_CONFIG_CUSTOM, "-mapx", "polyTagMapX", "TagMapX",
+	DEF_TAG_AXIS, Tk_Offset(PolygonTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &XAxisFlagsOption},
+    {TK_CONFIG_CUSTOM, "-mapy", "polyTagMapY", "TagMapY",
+	DEF_TAG_AXIS, Tk_Offset(PolygonTag, axisFlags),
+	TK_CONFIG_DONT_SET_DEFAULT, &YAxisFlagsOption},
     {TK_CONFIG_BITMAP, "-stipple", "polyTagStipple", "TagStipple",
 	(char *)NULL, Tk_Offset(PolygonTag, stipple), TK_CONFIG_NULL_OK},
     {TK_CONFIG_PIXELS, "-xoffset", "polyTagXOffset", "TagXOffset",
@@ -421,18 +466,18 @@ GetExprValue(interp, expr, valuePtr)
     length = strlen(expr);
     if ((expr[0] == 'I') && (strcmp(expr, "Inf") == 0)) {
 	*valuePtr = Blt_posInfinity;	/* Elastic upper bound */
-    } else if ((expr[0] == '-') && (expr[1] == 'I') && 
-	       (strcmp(expr, "-Inf") == 0)) {
+    } else if ((expr[0] == '-') && (expr[1] == 'I') &&
+	(strcmp(expr, "-Inf") == 0)) {
 	*valuePtr = Blt_negInfinity;	/* Elastic lower bound */
     } else if ((expr[0] == 'm') && (length > 1) &&
-	       (strncasecmp(expr, "minAxisValue", length) == 0)) {
+	(strncasecmp(expr, "minAxisValue", length) == 0)) {
 	*valuePtr = Blt_negInfinity;	/* Elastic lower bound */
     } else if ((expr[0] == 'm') && (length > 1) &&
-	       (strncasecmp(expr, "maxAxisValue", length) == 0)) {
+	(strncasecmp(expr, "maxAxisValue", length) == 0)) {
 	*valuePtr = Blt_posInfinity;	/* Elastic upper bound */
     } else if (Tcl_ExprDouble(interp, expr, valuePtr) != TCL_OK) {
 	Tcl_AppendResult(interp, "bad expression \"", expr, "\"",
-			 (char *)NULL);
+	    (char *)NULL);
 	return TCL_ERROR;
     }
     return TCL_OK;
@@ -445,7 +490,7 @@ GetExprValue(interp, expr, valuePtr)
  *
  * 	Convert the double precision value into its string representation.
  *	The only reason this routine is used in instead of sprintf, is to
- *	handle the "elastic" bounds.  That is, convert the values 
+ *	handle the "elastic" bounds.  That is, convert the values
  *	MAX_DBL_VALUE and MIN_DBL_VALUE into "maxvalue" and "minvalue"
  *	respectively.
  *
@@ -517,11 +562,11 @@ ConfigureBitmap(graphPtr, tagPtr)
 	}
 
 	/*
-	 * Note that this is *not* a shared GC.  GCTileStipXOrigin and 
+	 * Note that this is *not* a shared GC.  GCTileStipXOrigin and
 	 * GCTileStipYOrigin may change when displayed.
 	 */
-	newGC = XCreateGC(graphPtr->display, Tk_WindowId(graphPtr->tkwin), 
-		gcMask, &gcValues);
+	newGC = XCreateGC(graphPtr->display, Tk_WindowId(graphPtr->tkwin),
+	    gcMask, &gcValues);
 	if (bmTagPtr->gc != NULL) {
 	    XFreeGC(graphPtr->display, bmTagPtr->gc);
 	}
@@ -575,7 +620,8 @@ LayoutBitmap(graphPtr, tagPtr)
 	return;
     }
     Tk_SizeOfBitmap(graphPtr->display, bmTagPtr->bitmap, &width, &height);
-    win = Blt_WinPt(graphPtr, bmTagPtr->coordArr[0], bmTagPtr->coordArr[1]);
+    win = Blt_TransformPt(graphPtr, bmTagPtr->coordArr[0],
+	bmTagPtr->coordArr[1], bmTagPtr->axisFlags);
     bmPos = Blt_TranslateBoxCoords(win.x, win.y, width, height,
 	bmTagPtr->anchor);
     bmTagPtr->x = (bmPos.x + bmTagPtr->xOffset);
@@ -659,8 +705,8 @@ DisplayBitmap(graphPtr, tagPtr)
 
 	    Tk_SizeOfBitmap(graphPtr->display, bmTagPtr->bitmap, &width,
 		&height);
-	    Blt_GetBoundingBox(width, height, bmTagPtr->theta, (int *)&width, 
-			       (int *)&height, pointArr);
+	    Blt_GetBoundingBox(width, height, bmTagPtr->theta, &width, &height,
+		pointArr);
 	    for (i = 0; i < 4; i++) {
 		pointArr[i].x += bmTagPtr->x + (bmTagPtr->width / 2);
 		pointArr[i].y += bmTagPtr->y + (bmTagPtr->height / 2);
@@ -830,8 +876,8 @@ ConfigureText(graphPtr, tagPtr)
 	 * GCTileStipXOrigin, GCTileStipYOrigin, GCStipple values will be
 	 * changed when the layout is recalculated.
 	 */
-	newGC = XCreateGC(graphPtr->display, Tk_WindowId(graphPtr->tkwin), 
-		gcMask, &gcValues);
+	newGC = XCreateGC(graphPtr->display, Tk_WindowId(graphPtr->tkwin),
+	    gcMask, &gcValues);
 	if (textTagPtr->gc != NULL) {
 	    XFreeGC(graphPtr->display, textTagPtr->gc);
 	}
@@ -879,7 +925,8 @@ LayoutText(graphPtr, tagPtr)
     if (textTagPtr->text == NULL) {
 	return;
     }
-    win = Blt_WinPt(graphPtr, textTagPtr->coordArr[0], textTagPtr->coordArr[1]);
+    win = Blt_TransformPt(graphPtr, textTagPtr->coordArr[0],
+	textTagPtr->coordArr[1], textTagPtr->axisFlags);
     textPos = Blt_TranslateTextCoords(textTagPtr->fontPtr, textTagPtr->text,
 	win.x, win.y, textTagPtr->anchor);
     /* Save page coordinates */
@@ -1124,7 +1171,6 @@ CreateText()
     register Tag *tagPtr;
 
     tagPtr = (Tag *)calloc(1, sizeof(TextTag));
-
     if (tagPtr != NULL) {
 	tagPtr->configSpecs = textConfigSpecs;
 	tagPtr->configProc = ConfigureText;
@@ -1287,8 +1333,8 @@ LayoutWindow(graphPtr, tagPtr)
     if (winTagPtr->child != (Tk_Window)NULL) {
 	XPoint win;		/* Calculated window coordinates of window */
 
-	win = Blt_WinPt(graphPtr, winTagPtr->coordArr[0],
-	    winTagPtr->coordArr[1]);
+	win = Blt_TransformPt(graphPtr, winTagPtr->coordArr[0],
+	    winTagPtr->coordArr[1], winTagPtr->axisFlags);
 	win = Blt_TranslateBoxCoords(win.x, win.y, winTagPtr->width,
 	    winTagPtr->height, winTagPtr->anchor);
 	winTagPtr->x = win.x + winTagPtr->xOffset;
@@ -1379,7 +1425,6 @@ CreateWindow()
     register Tag *tagPtr;
 
     tagPtr = (Tag *)calloc(1, sizeof(WindowTag));
-
     if (tagPtr != NULL) {
 	tagPtr->configSpecs = windowConfigSpecs;
 	tagPtr->configProc = ConfigureWindow;
@@ -1473,14 +1518,14 @@ LayoutLine(graphPtr, tagPtr)
 	return;			/* Too few points */
     }
     pointArr = (XPoint *)malloc((lineTagPtr->numCoords / 2) * sizeof(XPoint));
-
     if (pointArr == NULL) {
 	return;			/* Can't allocate new point array */
     }
     numPoints = 0;
     for (i = 0; i < lineTagPtr->numCoords; i += 2) {
-	pointArr[numPoints] = Blt_WinPt(graphPtr, lineTagPtr->coordArr[i],
-	    lineTagPtr->coordArr[i + 1]);
+	pointArr[numPoints] = Blt_TransformPt(graphPtr,
+	    lineTagPtr->coordArr[i], lineTagPtr->coordArr[i + 1],
+	    lineTagPtr->axisFlags);
 	pointArr[numPoints].x += lineTagPtr->xOffset;
 	pointArr[numPoints].y += lineTagPtr->yOffset;
 	numPoints++;
@@ -1529,32 +1574,23 @@ PrintLine(graphPtr, tagPtr)
     Tag *tagPtr;
 {
     LineTag *lineTagPtr = (LineTag *)tagPtr;
-    register int i;
 
-    if (lineTagPtr->numPoints < 2) {
-	return;
-    }
-    Blt_LineWidthToPostScript(graphPtr, lineTagPtr->lineWidth);
-    Blt_ForegroundToPostScript(graphPtr, lineTagPtr->fgColorPtr);
-    if (lineTagPtr->dashes > 0) {
-	Blt_LineDashesToPostScript(graphPtr, lineTagPtr->dashes);
-
-	Tcl_AppendResult(graphPtr->interp, "/DashesProc {\ngsave\n",
-	    (char *)NULL);
-	Blt_BackgroundToPostScript(graphPtr, lineTagPtr->bgColorPtr);
-	Blt_LineDashesToPostScript(graphPtr, 0);
-	Tcl_AppendResult(graphPtr->interp, "stroke grestore\n} def\n",
-	    (char *)NULL);
-    } else {
-	Tcl_AppendResult(graphPtr->interp, "/DashesProc {} def\n",
-	    (char *)NULL);
-    }
-    sprintf(graphPtr->scratchPtr, "/curX %d def\n/curY %d def\n",
-	lineTagPtr->pointArr[0].x, lineTagPtr->pointArr[0].y);
-    Tcl_AppendResult(graphPtr->interp, graphPtr->scratchPtr, (char *)NULL);
-    for (i = 1; i < lineTagPtr->numPoints; i++) {
-	sprintf(graphPtr->scratchPtr, "%d %d DrawLine\n",
-	    lineTagPtr->pointArr[i].x, lineTagPtr->pointArr[i].y);
+    if (lineTagPtr->numPoints > 1) {
+	Blt_LineWidthToPostScript(graphPtr, lineTagPtr->lineWidth);
+	Blt_ForegroundToPostScript(graphPtr, lineTagPtr->fgColorPtr);
+	if (lineTagPtr->dashes > 0) {
+	    Blt_LineDashesToPostScript(graphPtr, lineTagPtr->dashes);
+	    Tcl_AppendResult(graphPtr->interp, "/DashesProc {\ngsave\n",
+		(char *)NULL);
+	    Blt_BackgroundToPostScript(graphPtr, lineTagPtr->bgColorPtr);
+	    Blt_LineDashesToPostScript(graphPtr, 0);
+	    Tcl_AppendResult(graphPtr->interp, "stroke grestore\n} def\n",
+		(char *)NULL);
+	} else {
+	    Tcl_AppendResult(graphPtr->interp, "/DashesProc {} def\n",
+		(char *)NULL);
+	}
+	Blt_PrintLine(graphPtr, lineTagPtr->pointArr, lineTagPtr->numPoints);
 	Tcl_AppendResult(graphPtr->interp, graphPtr->scratchPtr, (char *)NULL);
     }
 }
@@ -1621,7 +1657,6 @@ CreateLine()
     register Tag *tagPtr;
 
     tagPtr = (Tag *)calloc(1, sizeof(LineTag));
-
     if (tagPtr != NULL) {
 	tagPtr->configSpecs = lineConfigSpecs;
 	tagPtr->configProc = ConfigureLine;
@@ -1723,14 +1758,14 @@ LayoutPolygon(graphPtr, tagPtr)
     }
     numPoints = (polyTagPtr->numCoords / 2) + 1;
     pointArr = (XPoint *)malloc(numPoints * sizeof(XPoint));
-
     if (pointArr == NULL) {
 	return;			/* Can't allocate point array */
     }
     numPoints = 0;
     for (i = 0; i < polyTagPtr->numCoords; i += 2) {
-	pointArr[numPoints] = Blt_WinPt(graphPtr, polyTagPtr->coordArr[i],
-	    polyTagPtr->coordArr[i + 1]);
+	pointArr[numPoints] = Blt_TransformPt(graphPtr,
+	    polyTagPtr->coordArr[i], polyTagPtr->coordArr[i + 1],
+	    polyTagPtr->axisFlags);
 	pointArr[numPoints].x += polyTagPtr->xOffset;
 	pointArr[numPoints].y += polyTagPtr->yOffset;
 	numPoints++;
@@ -1739,8 +1774,8 @@ LayoutPolygon(graphPtr, tagPtr)
     /*
      * Make certain the polygon is closed
      */
-    pointArr[numPoints++] = Blt_WinPt(graphPtr, polyTagPtr->coordArr[0],
-	polyTagPtr->coordArr[1]);
+    pointArr[numPoints++] = Blt_TransformPt(graphPtr, polyTagPtr->coordArr[0],
+	polyTagPtr->coordArr[1], polyTagPtr->axisFlags);
     if (polyTagPtr->pointArr != NULL) {
 	free((char *)polyTagPtr->pointArr);
     }
@@ -1861,7 +1896,6 @@ CreatePolygon()
     register Tag *tagPtr;
 
     tagPtr = (Tag *)calloc(1, sizeof(PolygonTag));
-
     if (tagPtr != NULL) {
 	tagPtr->configSpecs = polygonConfigSpecs;
 	tagPtr->configProc = ConfigurePolygon;
@@ -1962,18 +1996,18 @@ GetTagCoords(interp, tagPtr, coordList)
 	    goto error;
 	}
 	switch (tagPtr->type) {
-	  case LINE_TAG_TYPE:
+	case LINE_TAG_TYPE:
 	    minArgs = 4, maxArgs = 0;
 	    break;
-	  case POLYGON_TAG_TYPE:
+	case POLYGON_TAG_TYPE:
 	    minArgs = 6, maxArgs = 0;
 	    break;
-	  case WINDOW_TAG_TYPE:
-	  case BITMAP_TAG_TYPE:
-	  case TEXT_TAG_TYPE:
+	case WINDOW_TAG_TYPE:
+	case BITMAP_TAG_TYPE:
+	case TEXT_TAG_TYPE:
 	    minArgs = 2, maxArgs = 2;
 	    break;
-	  default:
+	default:
 	    interp->result = "Unknown tag type";
 	    goto error;
 	}
@@ -1986,6 +2020,10 @@ GetTagCoords(interp, tagPtr, coordList)
 	    goto error;
 	}
 	coordArr = (double *)malloc((unsigned int)numExprs * sizeof(double));
+	if (coordArr == NULL) {
+	    interp->result = "can't allocate coordinate array";
+	    goto error;
+	}
 	coordPtr = coordArr;
 	for (i = 0; i < numExprs; i++) {
 	    if (GetExprValue(interp, exprArr[i], &value) != TCL_OK) {
@@ -2069,7 +2107,7 @@ TagIds(graphPtr, argc, argv)
     Blt_ListEntry *entryPtr;
 
     for (entryPtr = Blt_FirstListEntry(&(graphPtr->tagList)); entryPtr != NULL;
-	 entryPtr = Blt_NextListEntry(entryPtr)) {
+	entryPtr = Blt_NextListEntry(entryPtr)) {
 	tagPtr = (Tag *)Blt_GetListValue(entryPtr);
 	if ((argc == 0) || (Tcl_StringMatch(tagPtr->id, argv[0]))) {
 	    Tcl_AppendElement(graphPtr->interp, tagPtr->id);
@@ -2182,7 +2220,8 @@ CreateTag(graphPtr, argc, argv)
     char tagId[80];
     Blt_ListEntry *listPtr;
     Tcl_HashEntry *hashPtr;
-    int type, dummy;
+    int dummy;
+    TagClassType type;
 
     type = GetTagClassType(argv[0]);
     if (type < 0) {
@@ -2192,28 +2231,31 @@ CreateTag(graphPtr, argc, argv)
     }
     tagPtr = NULL;
     switch (type) {
-	case LINE_TAG_TYPE:
-	    tagPtr = CreateLine();
-	    break;
-	case WINDOW_TAG_TYPE:
-	    tagPtr = CreateWindow();
-	    break;
-	case BITMAP_TAG_TYPE:
-	    tagPtr = CreateBitmap();
-	    break;
-	case POLYGON_TAG_TYPE:
-	    tagPtr = CreatePolygon();
-	    break;
-	case TEXT_TAG_TYPE:
-	    tagPtr = CreateText();
-	    break;
+    case LINE_TAG_TYPE:
+	tagPtr = CreateLine();
+	break;
+    case WINDOW_TAG_TYPE:
+	tagPtr = CreateWindow();
+	break;
+    case BITMAP_TAG_TYPE:
+	tagPtr = CreateBitmap();
+	break;
+    case POLYGON_TAG_TYPE:
+	tagPtr = CreatePolygon();
+	break;
+    case TEXT_TAG_TYPE:
+	tagPtr = CreateText();
+	break;
+    default:
+	graphPtr->interp->result = "unknown tag type";
+	return TCL_ERROR;
     }
     if (tagPtr == NULL) {
-	Tcl_AppendResult(graphPtr->interp, "can't allocate new tag: ",
-	    Tcl_PosixError(graphPtr->interp), (char *)NULL);
+	graphPtr->interp->result = "can't allocate new tag";
 	return TCL_ERROR;
     }
     tagPtr->type = (TagClassType) type;
+    tagPtr->axisFlags = (AXIS_X1_FLAG | AXIS_Y1_FLAG);
     tagPtr->typeProc = TypeOfTag;
     sprintf(tagId, "TAG%d", graphPtr->nextTagId++);
     tagPtr->id = Tk_GetUid(tagId);
@@ -2467,8 +2509,8 @@ Blt_TagCmd(graphPtr, argc, argv)
 	    return TCL_ERROR;
 	}
 	result = DeleteTag(graphPtr, argc - 3, argv + 3);
-    } else if ((c == 't') && (length > 1) && 
-	       (strncmp(argv[2], "type", length) == 0)) {
+    } else if ((c == 't') && (length > 1) &&
+	(strncmp(argv[2], "type", length) == 0)) {
 	if (argc != 4) {
 	    Tcl_AppendResult(graphPtr->interp, "wrong # args: should be \"",
 		argv[0], " tag type tagId\"", (char *)NULL);
