@@ -4,7 +4,7 @@
  *	This module implements a graph widget for the
  *	Tk toolkit.
  *
- * Copyright 1991-1993 by AT&T Bell Laboratories.
+ * Copyright 1991-1994 by AT&T Bell Laboratories.
  * Permission to use, copy, modify, and distribute this software
  * and its documentation for any purpose and without fee is hereby
  * granted, provided that the above copyright notice appear in all
@@ -43,7 +43,7 @@
  *
  * 9) Make sure reasonable defaults show for configuration items
  *
- * 10) Rubberbox for zooming??? 
+ * 10) Rubberbox for zooming???
  */
 
 #include "blt.h"
@@ -54,29 +54,16 @@
 #include <X11/Xatom.h>
 
 #ifndef GRAPH_VERSION
-#define GRAPH_VERSION "2.1"
+#define GRAPH_VERSION "3.0"
 #endif
 
 #define TRUE  1
 #define FALSE 0
 
-#ifdef hpux
-#define BLACK		"black"
-#define WHITE		"white"
-#define LIGHTGREY       "lightgrey"
-#define BISQUE1		"bisque1"
-#else
-#define BLACK		"#000000"
-#define WHITE		"#ffffff"
-#define LIGHTGREY       "#d3d3d3"
-#define BISQUE1		"#ffe4c4"
-#endif
-
 static char *classNames[] =
 {
     "Blt_graph", "Blt_barchart"
 };
-
 static int configFlags[] =
 {
     XYGRAPH_MASK, BARCHART_MASK
@@ -92,17 +79,14 @@ static int configFlags[] =
 #define DEF_GRAPH_FONT		"*-Helvetica-Bold-R-Normal-*-120-*"
 #define DEF_GRAPH_HEIGHT	"400"
 #define DEF_GRAPH_WIDTH		"400"
-#define DEF_GRAPH_TITLE		(char *)NULL
 #define DEF_EXT_BORDER_WIDTH   	(char *)NULL
 #define DEF_EXT_BORDER_RELIEF	"flat"
-#define DEF_PLOT_BG_COLOR	LIGHTGREY
+#define DEF_PLOT_BG_COLOR	ANTIQUEWHITE1
 #define DEF_PLOT_BG_MONO	WHITE
 #define DEF_INT_RELIEF_COLOR	"sunken"
 #define DEF_INT_RELIEF_MONO	"flat"
 #define DEF_INT_BW_MONO       	(char *)NULL
 #define DEF_INT_BW_COLOR      	"2"
-#define DEF_INT_BG_MONO	 	WHITE
-#define DEF_INT_BG_COLOR       	LIGHTGREY
 
 static Tk_ConfigSpec configSpecs[] =
 {
@@ -170,10 +154,6 @@ static Tk_ConfigSpec configSpecs[] =
 	(char *)NULL, Tk_Offset(Graph, topMargin), ALL_MASK},
     {TK_CONFIG_PIXELS, "-width", "width", "Width",
 	DEF_GRAPH_WIDTH, Tk_Offset(Graph, reqWidth), ALL_MASK},
-    {TK_CONFIG_STRING, "-xtitle", "xTitle", "Title",
-	"X", Tk_Offset(Graph, xTitle), ALL_MASK | TK_CONFIG_NULL_OK},
-    {TK_CONFIG_STRING, "-ytitle", "yTitle", "Title",
-	"Y", Tk_Offset(Graph, yTitle), ALL_MASK | TK_CONFIG_NULL_OK},
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
 };
 
@@ -182,6 +162,8 @@ double Blt_negInfinity, Blt_posInfinity;
 extern int Blt_CreateAxis _ANSI_ARGS_((Graph *, AxisType, int));
 extern int Blt_AxisCmd _ANSI_ARGS_((Graph *, GraphAxis *, int, char **, int));
 extern void Blt_ComputeAxes _ANSI_ARGS_((Graph *));
+extern void Blt_UpdateAxisBackgrounds _ANSI_ARGS_((Graph *, XColor *));
+extern int Blt_ComputeLayout _ANSI_ARGS_((Graph *));
 
 extern int Blt_LegendCmd _ANSI_ARGS_((Graph *, int, char **));
 extern int Blt_CreateLegend _ANSI_ARGS_((Graph *));
@@ -292,7 +274,8 @@ GraphCoords(graphPtr, argc, argv)
     char **argv;
 {
     int winX, winY;		/* Integer window coordinates representation */
-    char string[200];
+    char string[TCL_DOUBLE_SPACE + 1];
+    double value;
 
     if (argc != 4) {
 	Tcl_AppendResult(graphPtr->interp, "wrong # args: should be \"",
@@ -306,9 +289,12 @@ GraphCoords(graphPtr, argc, argv)
     /*
      * Perform the reverse transformation from window coordinates to data
      */
-    Tcl_PrintDouble(graphPtr->interp, Blt_WorldX(graphPtr, winX), string);
+    value = Blt_InvTransform(graphPtr->axisArr[X1], winX);
+    Tcl_PrintDouble(graphPtr->interp, value, string);
     Tcl_AppendElement(graphPtr->interp, string);
-    Tcl_PrintDouble(graphPtr->interp, Blt_WorldY(graphPtr, winY), string);
+
+    value = Blt_InvTransform(graphPtr->axisArr[Y1], winY);
+    Tcl_PrintDouble(graphPtr->interp, value, string);
     Tcl_AppendElement(graphPtr->interp, string);
     return TCL_OK;
 }
@@ -323,20 +309,21 @@ GraphCoords(graphPtr, argc, argv)
  *
  * Results:
  *	Returns a standard Tcl result.  interp->result contains
- *	the list of the graph coordinates. If an error occurred 
- *	while parsing the window positions, TCL_ERROR is returned, 
+ *	the list of the graph coordinates. If an error occurred
+ *	while parsing the window positions, TCL_ERROR is returned,
  *	then interp->result will contain an error message.
  *
  *--------------------------------------------------------------
  */
 static int
-WindowCoords (graphPtr, argc, argv)
+WindowCoords(graphPtr, argc, argv)
     Graph *graphPtr;		/* Graph widget record */
     int argc;
     char **argv;
 {
     double x, y;
-    char string[200];
+    char string[TCL_DOUBLE_SPACE + 1];
+    int winX, winY;
 
     if (argc != 4) {
 	Tcl_AppendResult(graphPtr->interp, "wrong # args: should be \"",
@@ -350,9 +337,9 @@ WindowCoords (graphPtr, argc, argv)
     /*
      * Perform the transformation from window coordinates to data
      */
-    sprintf (string, "%d", Blt_WinX(graphPtr, x));
-    Tcl_AppendElement(graphPtr->interp, string);
-    sprintf (string, "%d", Blt_WinY(graphPtr, y));
+    winX = Blt_Transform(graphPtr->axisArr[X1], x);
+    winY = Blt_Transform(graphPtr->axisArr[Y1], y);
+    sprintf(string, "%d %d", winX, winY);
     Tcl_AppendElement(graphPtr->interp, string);
     return TCL_OK;
 }
@@ -383,6 +370,8 @@ DestroyGraph(clientData)
     Tcl_HashSearch cursor;
     Element *elemPtr;
     Tag *tagPtr;
+    register int i;
+    GraphAxis *axisPtr;
 
     /*
      * Destroy the individual components of the graph: elements, tags, X and Y
@@ -403,8 +392,11 @@ DestroyGraph(clientData)
     Tcl_DeleteHashTable(&(graphPtr->tagTable));
     Blt_ClearList(&(graphPtr->tagList));
 
-    (*graphPtr->X->destroyProc) (graphPtr, graphPtr->X);
-    (*graphPtr->Y->destroyProc) (graphPtr, graphPtr->Y);
+    for (i = 0; i < 4; i++) {
+	axisPtr = graphPtr->axisArr[i];
+	(*axisPtr->destroyProc) (graphPtr, axisPtr);
+    }
+
     (*graphPtr->legendPtr->destroyProc) (graphPtr);
     (*graphPtr->postscript->destroyProc) (graphPtr);
     (*graphPtr->crosshairs->destroyProc) (graphPtr);
@@ -418,7 +410,7 @@ DestroyGraph(clientData)
 	Tk_FreeGC(graphPtr->display, graphPtr->extBgGC);
     }
     Tk_FreeOptions(configSpecs, (char *)graphPtr, graphPtr->display,
-	configFlags[(int)graphPtr->type]);
+	configFlags[graphPtr->type]);
     free((char *)graphPtr);
 }
 
@@ -450,15 +442,15 @@ CreateGraph(interp, parent, pathName, type)
     register Graph *graphPtr;
     Tk_Window tkwin;
     int flags = configFlags[(int)type];
+    register int i;
 
     tkwin = Tk_CreateWindowFromPath(interp, parent, pathName, (char *)NULL);
     if (tkwin == (Tk_Window)NULL) {
 	return (Graph *) NULL;
     }
     graphPtr = (Graph *)calloc(1, sizeof(Graph));
-
     if (graphPtr == (Graph *)NULL) {
-	interp->result = "Can't allocate graph structure";
+	interp->result = "can't allocate graph structure";
 	return (Graph *) NULL;
     }
     Tk_SetClass(tkwin, classNames[(int)type]);
@@ -482,10 +474,13 @@ CreateGraph(interp, parent, pathName, type)
      * Axes and tags (text, bitmap) need private GCs, so create the window now.
      */
     Tk_MakeWindowExist(tkwin);
-    if (Blt_CreateAxis(graphPtr, X_AXIS_TYPE, flags) != TCL_OK) {
-	goto error;
+
+    for (i = 0; i < 4; i++) {
+	if (Blt_CreateAxis(graphPtr, (AxisType)i, flags) != TCL_OK) {
+	    goto error;
+	}
     }
-    if (Blt_CreateAxis(graphPtr, Y_AXIS_TYPE, flags) != TCL_OK) {
+    if (Blt_CreatePostScript(graphPtr) != TCL_OK) {
 	goto error;
     }
     if (Blt_CreateCrosshairs(graphPtr) != TCL_OK) {
@@ -494,9 +489,6 @@ CreateGraph(interp, parent, pathName, type)
     if (Blt_CreateLegend(graphPtr) != TCL_OK) {
 	goto error;
     }
-    if (Blt_CreatePostScript(graphPtr) != TCL_OK) {
-	goto error;
-    }
     /* Compute the initial axis ticks and labels */
     Blt_ComputeAxes(graphPtr);
     Tk_CreateEventHandler(graphPtr->tkwin, ExposureMask | StructureNotifyMask,
@@ -554,7 +546,7 @@ ConfigureGraph(graphPtr, argc, argv, flags)
 	    Tk_PathName(graphPtr->tkwin), "\"", (char *)NULL);
 	return TCL_ERROR;
     }
-    Tk_GeometryRequest(graphPtr->tkwin, graphPtr->reqWidth, 
+    Tk_GeometryRequest(graphPtr->tkwin, graphPtr->reqWidth,
 	graphPtr->reqHeight);
     Tk_SetInternalBorder(graphPtr->tkwin, graphPtr->extBWidth);
     Tk_SetBackgroundFromBorder(graphPtr->tkwin, graphPtr->border);
@@ -567,7 +559,7 @@ ConfigureGraph(graphPtr, argc, argv, flags)
     Blt_UpdateAxisBackgrounds(graphPtr, borderColor);
 
     /*
-     * Create GCs for interior and exterior regions, and a background 
+     * Create GCs for interior and exterior regions, and a background
      * GC for clearing the margins with XFillRectangle
      */
     gcValues.foreground = graphPtr->fgColorPtr->pixel;
@@ -592,10 +584,10 @@ ConfigureGraph(graphPtr, argc, argv, flags)
 	Tk_FreeGC(graphPtr->display, graphPtr->intGC);
     }
     graphPtr->intGC = newGC;
-    /* 
-     * Reconfigure crosshairs in case the plot background color changed 
+    /*
+     * Reconfigure crosshairs in case the plot background color changed
      */
-    (*graphPtr->crosshairs->configProc)(graphPtr);
+    (*graphPtr->crosshairs->configProc) (graphPtr);
     /*
      * Title font, title strings, or margins may have changed layout
      */
@@ -637,11 +629,11 @@ GraphCmd(clientData, interp, argc, argv)
 	return TCL_ERROR;
     }
     if (strcmp(argv[0], "blt_graph") == 0) {
-	type = GRAPH_TYPE;
+	type = GRAPH;
     } else if (strcmp(argv[0], "blt_barchart") == 0) {
-	type = BARCHART_TYPE;
+	type = BARCHART;
     } else if (strcmp(argv[0], "blt_contour") == 0) {
-	type = CONTOUR_TYPE;
+	type = CONTOUR;
     } else {
 	Tcl_AppendResult(interp, "unknown graph-creation command \"", argv[0],
 	    "\"", (char *)NULL);
@@ -693,6 +685,7 @@ GraphWidgetCmd(clientData, interp, argc, argv)
     int length;
     int flags;
     GraphClassType type;
+    GraphAxis *axisPtr;
 
     if (argc < 2) {
 	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
@@ -724,10 +717,22 @@ GraphWidgetCmd(clientData, interp, argc, argv)
 	result = Blt_ElementCmd(graphPtr, argc, argv);
     } else if ((c == 't') && (strncmp(argv[1], "tag", length) == 0)) {
 	result = Blt_TagCmd(graphPtr, argc, argv);
-    } else if ((c == 'x') && (strncmp(argv[1], "xaxis", length) == 0)) {
-	result = Blt_AxisCmd(graphPtr, graphPtr->X, argc, argv, flags);
-    } else if ((c == 'y') && (strncmp(argv[1], "yaxis", length) == 0)) {
-	result = Blt_AxisCmd(graphPtr, graphPtr->Y, argc, argv, flags);
+    } else if ((c == 'x') && (length > 1) &&
+	(strncmp(argv[1], "xaxis", length) == 0)) {
+	axisPtr = graphPtr->axisArr[X1];
+	result = Blt_AxisCmd(graphPtr, axisPtr, argc, argv, flags);
+    } else if ((c == 'x') && (length > 1) &&
+	(strncmp(argv[1], "x2axis", length) == 0)) {
+	axisPtr = graphPtr->axisArr[X2];
+	result = Blt_AxisCmd(graphPtr, axisPtr, argc, argv, flags);
+    } else if ((c == 'y') && (length > 1) &&
+	(strncmp(argv[1], "yaxis", length) == 0)) {
+	axisPtr = graphPtr->axisArr[Y1];
+	result = Blt_AxisCmd(graphPtr, axisPtr, argc, argv, flags);
+    } else if ((c == 'y') && (length > 1) &&
+	(strncmp(argv[1], "y2axis", length) == 0)) {
+	axisPtr = graphPtr->axisArr[Y2];
+	result = Blt_AxisCmd(graphPtr, axisPtr, argc, argv, flags);
     } else if ((c == 'l') && (length > 1) &&
 	(strncmp(argv[1], "legend", length) == 0)) {
 	result = Blt_LegendCmd(graphPtr, argc, argv);
@@ -742,14 +747,14 @@ GraphWidgetCmd(clientData, interp, argc, argv)
 	(strncmp(argv[1], "transform", length) == 0)) {
 	result = WindowCoords(graphPtr, argc, argv);
     } else if ((c == 'p') && (strncmp(argv[1], "postscript", length) == 0)) {
-	result = (*graphPtr->postscript->printProc)(graphPtr, argc, argv);
+	result = (*graphPtr->postscript->printProc) (graphPtr, argc, argv);
     } else if ((c == 'p') && (length > 1) &&
 	(strncmp(argv[1], "psconfigure", length) == 0)) {
-	result = (*graphPtr->postscript->configProc)(graphPtr, argc, argv);
+	result = (*graphPtr->postscript->configProc) (graphPtr, argc, argv);
     } else {
 	Tcl_AppendResult(interp, "bad option \"", argv[1], "\": should be\
  configure, crosshairs, element, invtransform, legend, postscript,\
- psconfigure, tag, transform, xaxis, or yaxis", (char *)NULL);
+ psconfigure, tag, transform, xaxis, yaxis, or y2axis", (char *)NULL);
     }
     Tk_Release((ClientData)graphPtr);
     return result;
@@ -784,11 +789,16 @@ Blt_LayoutGraph(graphPtr)
     Tag *tagPtr;
 
     /*
-     * Layout both X and Y axis
+     * Layout axes
      */
     if (graphPtr->flags & LAYOUT_ALL) {
-	(*graphPtr->X->layoutProc) (graphPtr);
-	(*graphPtr->Y->layoutProc) (graphPtr);
+	GraphAxis *axisPtr;
+	register int i;
+
+	for (i = 0; i < 4; i++) {
+	    axisPtr = graphPtr->axisArr[i];
+	    (*axisPtr->layoutProc) (graphPtr, axisPtr);
+	}
     }
     /*
      * Layout elements
@@ -905,8 +915,8 @@ DisplayTags(graphPtr)
  * DisplayExterior --
  *
  * 	Draws the exterior region of the graph (axes, ticks, titles, etc)
- *	onto a pixmap. The interior region is defined by the given 
- *	rectangle structure. 
+ *	onto a pixmap. The interior region is defined by the given
+ *	rectangle structure.
  *
  *		X coordinate axis
  *		Y coordinate axis
@@ -917,7 +927,7 @@ DisplayTags(graphPtr)
  *
  * Returns:
  *	None.
- * 
+ *
  * Side Effects:
  *	Exterior of graph is displayed in its window.
  *
@@ -930,8 +940,9 @@ DisplayExterior(graphPtr, rectPtr)
 {
     int x, y;
     XRectangle rectArr[4];
+    GraphAxis *axisPtr;
     TextAttr textAttr;
-    int titlePos;
+    register int i;
 
     /*
      * Draw the four outer rectangles which encompass the plotting
@@ -939,15 +950,15 @@ DisplayExterior(graphPtr, rectPtr)
      */
 
     rectArr[0].x = rectArr[0].y = rectArr[3].x = rectArr[1].x = 0;
-    rectArr[0].width = rectArr[3].width = Tk_Width(graphPtr->tkwin);
+    rectArr[0].width = rectArr[3].width = graphPtr->width;
     rectArr[0].height = rectPtr->y;
     rectArr[3].y = rectPtr->y + rectPtr->height;
-    rectArr[3].height = Tk_Height(graphPtr->tkwin) - rectArr[3].y;
+    rectArr[3].height = graphPtr->height - rectArr[3].y;
     rectArr[2].y = rectArr[1].y = rectPtr->y;
     rectArr[1].width = rectPtr->x;
     rectArr[2].height = rectArr[1].height = rectPtr->height;
     rectArr[2].x = rectPtr->x + rectPtr->width;
-    rectArr[2].width = Tk_Width(graphPtr->tkwin) - rectArr[2].x;
+    rectArr[2].width = graphPtr->width - rectArr[2].x;
     XFillRectangles(graphPtr->display, graphPtr->canvas, graphPtr->extBgGC,
 	rectArr, 4);
 
@@ -958,43 +969,35 @@ DisplayExterior(graphPtr, rectPtr)
 	    rectPtr->height, graphPtr->intBWidth, graphPtr->intRelief);
     }
     /* Legend */
-    if (graphPtr->legendPtr->position.x == DEF_POSITION) {
+    if (graphPtr->legendPtr->useDefault) {
 	(*graphPtr->legendPtr->displayProc) (graphPtr);
     }
-    /* Graph and Axis titles */
+    /* Initialize text attributes for graph and axis titles */
+
     textAttr.theta = 0.0;
     textAttr.anchor = TK_ANCHOR_CENTER;
     textAttr.gc = graphPtr->extGC;
     textAttr.bgColorPtr = Tk_3DBorderColor(graphPtr->border);
     textAttr.fontPtr = graphPtr->fontPtr;
-    titlePos = graphPtr->extBWidth + TEXTHEIGHT(graphPtr->fontPtr);
 
-    x = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
     if (graphPtr->title != NULL) {
+	x = (graphPtr->extreme.x + graphPtr->origin.x) / 2;
+	y = graphPtr->extBWidth + TEXTHEIGHT(graphPtr->fontPtr);
 	Blt_DrawText(graphPtr->display, graphPtr->canvas, graphPtr->title,
-	    &textAttr, x, titlePos);
+	    &textAttr, x, y);
     }
-    if (graphPtr->xTitle != NULL) {
-	Blt_DrawText(graphPtr->display, graphPtr->canvas, graphPtr->xTitle,
-	    &textAttr, x, Tk_Height(graphPtr->tkwin) - titlePos);
+    /* Draw axes */
+    for (i = 0; i < 4; i++) {
+	axisPtr = graphPtr->axisArr[i];
+	if (axisPtr->mapped) {
+	    (*axisPtr->displayProc) (graphPtr, axisPtr, &textAttr);
+	}
     }
-    if (graphPtr->yTitle != NULL) {
-	y = (graphPtr->origin.y + graphPtr->extreme.y) / 2;
-	textAttr.theta = 90.0;
-	Blt_DrawText(graphPtr->display, graphPtr->canvas, graphPtr->yTitle,
-	    &textAttr, titlePos, y);
-    }
-    /* X and Y Axes */
-    (*graphPtr->X->displayProc) (graphPtr);
-    (*graphPtr->Y->displayProc) (graphPtr);
-
     /* Exterior 3D border */
-    if ((graphPtr->extRelief != TK_RELIEF_FLAT) &&
-	(graphPtr->extBWidth > 0)) {
+    if ((graphPtr->extRelief != TK_RELIEF_FLAT) && (graphPtr->extBWidth > 0)) {
 	Tk_Draw3DRectangle(graphPtr->display, graphPtr->canvas,
-	    graphPtr->border, 0, 0, Tk_Width(graphPtr->tkwin), 
-	    Tk_Height(graphPtr->tkwin), graphPtr->extBWidth, 
-	    graphPtr->extRelief);
+	    graphPtr->border, 0, 0, graphPtr->width, graphPtr->height,
+	    graphPtr->extBWidth, graphPtr->extRelief);
     }
 }
 
@@ -1036,8 +1039,10 @@ DisplayGraph(clientData)
     graphPtr->height = Tk_Height(graphPtr->tkwin);
 
     if (graphPtr->flags & LAYOUT_PENDING) {
-	if (Blt_ComputeLayout(graphPtr) != TCL_OK) {
-	    return;		/* Not enough room to plot graph */
+	if (graphPtr->flags & LAYOUT_ALL) {
+	    if (Blt_ComputeLayout(graphPtr) != TCL_OK) {
+		return;		/* Not enough room to plot graph */
+	    }
 	}
 	Blt_LayoutGraph(graphPtr);
     }
@@ -1054,16 +1059,16 @@ DisplayGraph(clientData)
      * Create a canvas the size of the window, filling it with the bg color
      */
     graphPtr->canvas = XCreatePixmap(graphPtr->display,
-	Tk_WindowId(graphPtr->tkwin), Tk_Width(graphPtr->tkwin), 
-	Tk_Height(graphPtr->tkwin), Tk_Depth(graphPtr->tkwin));
+	Tk_WindowId(graphPtr->tkwin), graphPtr->width, graphPtr->height,
+	Tk_Depth(graphPtr->tkwin));
     XFillRectangles(graphPtr->display, graphPtr->canvas, graphPtr->intGC,
 	&plotRect, 1);
     legendPtr = graphPtr->legendPtr;
-    if (legendPtr->position.x != DEF_POSITION) {
+    if (!legendPtr->useDefault) {
 	(*legendPtr->displayProc) (graphPtr);
     }
     /* Draw the graph elements and tags on the canvas */
-    if (graphPtr->type == GRAPH_TYPE) {
+    if (graphPtr->type == GRAPH) {
 	DisplayTags(graphPtr);
 	DisplayElements(graphPtr);
     } else {
@@ -1079,7 +1084,7 @@ DisplayGraph(clientData)
 	DisplayExterior(graphPtr, &plotRect);
 	XCopyArea(graphPtr->display, graphPtr->canvas,
 	    Tk_WindowId(graphPtr->tkwin), graphPtr->extGC, 0, 0,
-	    Tk_Width(graphPtr->tkwin), Tk_Height(graphPtr->tkwin), 0, 0);
+	    graphPtr->width, graphPtr->height, 0, 0);
     } else {
 	/* Draw just the interior plotting region */
 
@@ -1088,8 +1093,8 @@ DisplayGraph(clientData)
 	plotRect.width -= (graphPtr->intBWidth * 2);
 	plotRect.height -= (graphPtr->intBWidth * 2);
 	XCopyArea(graphPtr->display, graphPtr->canvas,
-	    Tk_WindowId(graphPtr->tkwin), graphPtr->extGC, plotRect.x, 
-	    plotRect.y, (unsigned int)plotRect.width, 
+	    Tk_WindowId(graphPtr->tkwin), graphPtr->extGC, plotRect.x,
+	    plotRect.y, (unsigned int)plotRect.width,
 	    (unsigned int)plotRect.height, plotRect.x, plotRect.y);
     }
     (*graphPtr->crosshairs->toggleProc) (graphPtr);
@@ -1105,14 +1110,14 @@ Blt_GraphInit(interp)
     Tk_Window tkwin;
 
     if (Blt_FindCmd(interp, "blt_graph", (ClientData *)NULL) == TCL_OK) {
-	Tcl_AppendResult(interp, "\"blt_graph\" command already exists", 
-			 (char *)NULL);
-	return TCL_ERROR;	
+	Tcl_AppendResult(interp, "\"blt_graph\" command already exists",
+	    (char *)NULL);
+	return TCL_ERROR;
     }
     tkwin = Tk_MainWindow(interp);
     if (tkwin == NULL) {
 	Tcl_AppendResult(interp, "\"blt_graph\" requires Tk", (char *)NULL);
-	return TCL_ERROR;	
+	return TCL_ERROR;
     }
     Tcl_SetVar2(interp, "blt_versions", "blt_graph", GRAPH_VERSION,
 	TCL_GLOBAL_ONLY);
@@ -1128,14 +1133,14 @@ Blt_BarchartInit(interp)
     Tk_Window tkwin;
 
     if (Blt_FindCmd(interp, "blt_barchart", (ClientData *)NULL) == TCL_OK) {
-	Tcl_AppendResult(interp, "\"blt_barchart\" command already exists", 
-			 (char *)NULL);
-	return TCL_ERROR;	
+	Tcl_AppendResult(interp, "\"blt_barchart\" command already exists",
+	    (char *)NULL);
+	return TCL_ERROR;
     }
     tkwin = Tk_MainWindow(interp);
     if (tkwin == NULL) {
 	Tcl_AppendResult(interp, "\"blt_barchart\" requires Tk", (char *)NULL);
-	return TCL_ERROR;	
+	return TCL_ERROR;
     }
     Tcl_SetVar2(interp, "blt_versions", "blt_barchart", GRAPH_VERSION,
 	TCL_GLOBAL_ONLY);
@@ -1143,4 +1148,3 @@ Blt_BarchartInit(interp)
 	(Tcl_CmdDeleteProc *)NULL);
     return TCL_OK;
 }
-
