@@ -1,7 +1,7 @@
 /*
  * bltGraph.h --
  *
- * Copyright 1991-1993 by AT&T Bell Laboratories.
+ * Copyright 1991-1994 by AT&T Bell Laboratories.
  * Permission to use, copy, modify, and distribute this software
  * and its documentation for any purpose and without fee is hereby
  * granted, provided that the above copyright notice appear in all
@@ -38,7 +38,7 @@
 /*
  * ----------------------------------------------------------------------
  *
- * 	MAX_DBL_VALUE and MIN_DBL_VALUE: 
+ * 	MAX_DBL_VALUE and MIN_DBL_VALUE:
  *
  *	There values are the largest and small values representable
  * 	on the graph or barchart.  They are used for the following
@@ -58,7 +58,7 @@
 #ifdef sun
 #define MAX_DBL_VALUE	1.7976931348623157E+308
 #define MIN_DBL_VALUE 	2.2250738585072014E-308
-#else 
+#else
 /* Start to go into punt formation. */
 #ifdef HUGE_VAL
 #define MAX_DBL_VALUE   HUGE_VAL
@@ -82,16 +82,16 @@
 #undef MAX
 #define MAX(a,b)	(((a)>(b))?(a):(b))
 
-/* 
+/*
  * ------------------------------------------------------------------------
  *
- * 	Replacement macros for math library functions, fabs, abs, rint, 
+ * 	Replacement macros for math library functions, fabs, abs, rint,
  *	and exp10.
- * 
+ *
  * 	Although many of these routines may be in the math library, they
- *	aren't used in libtcl.a or libtk.a.  This makes it impossible to 
- *	load BLT library as a shared object without requiring that the 
- *	math library be a shared object too.  Avoid the problem by 
+ *	aren't used in libtcl.a or libtk.a.  This makes it impossible to
+ *	load BLT library as a shared object without requiring that the
+ *	math library be a shared object too.  Avoid the problem by
  *	replacing the "exotic" math routines with macros.
  *
  * ------------------------------------------------------------------------
@@ -138,7 +138,6 @@
 #define XYGRAPH_MASK	TK_CONFIG_USER_BIT
 #define BARCHART_MASK	TK_CONFIG_USER_BIT << 1
 #define CONTOUR_MASK	TK_CONFIG_USER_BIT << 2
-#define PIECHART_MASK	TK_CONFIG_USER_BIT << 3
 #define ALL_MASK	(XYGRAPH_MASK | BARCHART_MASK)
 
 /*
@@ -151,10 +150,6 @@
 #define PADY    	2	/* Padding between labels */
 #define TEXTHEIGHT(f) 	((f)->ascent + (f)->descent)
 
-#define MAJOR_TICK 	0.020	/* Length of a major tick */
-#define MINOR_TICK 	0.015	/* Length of a minor (sub)tick */
-#define LABEL_TICK 	0.030	/* Distance from graph to start of label */
-
 #define DEF_POSITION -SHRT_MAX	/* Indicates that no position was specified */
 
 /*
@@ -181,9 +176,33 @@ typedef struct GraphCrosshairs GraphCrosshairs;
  * -------------------------------------------------------------------
  */
 typedef enum {
-    GRAPH_TYPE, BARCHART_TYPE, CONTOUR_TYPE, PIECHART_TYPE
+    GRAPH, BARCHART, CONTOUR
 } GraphClassType;
 
+/*
+ * -------------------------------------------------------------------
+ *
+ * 	TextAttr --
+ *
+ * 	Represents a convenience structure to hold text attributes
+ *	which determine how a text string is to be displayed on the
+ *	window, or drawn with PostScript commands.  The alternative
+ *	is to have drawing and printing routines with more parameters.
+ * 	This seems like a more efficient and less cumbersome way.
+ *
+ * -------------------------------------------------------------------
+ */
+typedef struct {
+    char *text;			/* Text string to be displayed  */
+    XFontStruct *fontPtr;	/* Font to use for string */
+    XColor *bgColorPtr;		/* Background color of string */
+    XColor *fgColorPtr;		/* Foreground color of string */
+    Tk_Anchor anchor;		/* Anchor type: used to position the string */
+    double theta;		/* Rotation of text in degrees. */
+    GC gc;			/* Graphics context to use when displaying the
+				 * string on the window */
+} TextAttr;
+
 /*
  * -------------------------------------------------------------------
  *
@@ -194,9 +213,50 @@ typedef enum {
  * -------------------------------------------------------------------
  */
 typedef enum AxisTypes {
-    X_AXIS_TYPE, Y_AXIS_TYPE, Z_AXIS_TYPE
+    X1, Y1, X2, Y2
 } AxisType;
 
+#define AXIS_X1_FLAG		(1<<X1)
+#define AXIS_Y1_FLAG		(1<<Y1)
+#define AXIS_X2_FLAG		(1<<X2)
+#define AXIS_Y2_FLAG		(1<<Y2)
+#define AXIS_X_MASK		(AXIS_X1_FLAG|AXIS_X2_FLAG)
+#define AXIS_Y_MASK		(AXIS_Y1_FLAG|AXIS_Y2_FLAG)
+#define AXIS_TYPE_MASK		(AXIS_Y_MASK|AXIS_X_MASK)
+#define AXIS_LIMIT_USER_BIT	0x10
+#define AXIS_LIMIT_MIN_MASK	(AXIS_LIMIT_USER_BIT << LMIN)
+#define AXIS_LIMIT_MAX_MASK	(AXIS_LIMIT_USER_BIT << LMAX)
+#define AXIS_LIMIT_MASK		(AXIS_LIMIT_MIN_MASK | AXIS_LIMIT_MAX_MASK)
+#define AXIS_MIN_AUTOSCALE(a) 	(((a)->flags & AXIS_LIMIT_MIN_MASK)==0)
+#define AXIS_MAX_AUTOSCALE(a) 	(((a)->flags & AXIS_LIMIT_MAX_MASK)==0)
+
+/*
+ * -------------------------------------------------------------------
+ *
+ * 	GraphAxis --
+ *
+ * 	Structure contains options controlling how the axis will be
+ * 	displayed.
+ *
+ * -------------------------------------------------------------------
+ */
+
+typedef void (AxisDisplayProc) _ANSI_ARGS_((Graph *, GraphAxis *, TextAttr *));
+typedef void (AxisPrintProc) _ANSI_ARGS_((Graph *, GraphAxis *, TextAttr *));
+typedef void (AxisLayoutProc) _ANSI_ARGS_((Graph *, GraphAxis *));
+typedef void (AxisDestroyProc) _ANSI_ARGS_((Graph *, GraphAxis *));
+
+struct GraphAxis {
+    unsigned int flags;		/* Axis flags */
+    int logScale;		/* If non-zero, scale values logarithmically */
+    int mapped;			/* If non-zero, display the axis */
+
+    AxisDisplayProc *displayProc;
+    AxisPrintProc *printProc;
+    AxisLayoutProc *layoutProc;
+    AxisDestroyProc *destroyProc;
+};
+
 /*
  * -------------------------------------------------------------------
  *
@@ -217,11 +277,12 @@ struct GraphLegend {
     int mapped;			/* Requested state of the legend, If non-zero,
 				 * legend is displayed */
     int width, height;		/* Dimensions of the legend */
-    XPoint position;		/* Window coordinates of legend positioning
+    XPoint anchorPos;		/* Window coordinates of legend positioning
 				 * point. Used in conjunction with the anchor
 				 * to determine the location of the legend. If
 				 * x or y are DEF_POSITION the legend is set
 				 * in the right margin */
+    int useDefault;		/* Draw the legend in the default location */
 
     LegendDisplayProc *displayProc;
     LegendPrintProc *printProc;
@@ -250,29 +311,6 @@ struct GraphPostScript {
     PostScriptDestroyProc *destroyProc;
 };
 
-/*
- * -------------------------------------------------------------------
- *
- * 	GraphAxis --
- *
- * 	Structure contains options controlling how the axis will be
- * 	displayed.
- *
- * -------------------------------------------------------------------
- */
-
-typedef void (AxisDisplayProc) _ANSI_ARGS_((Graph *));
-typedef void (AxisPrintProc) _ANSI_ARGS_((Graph *));
-typedef void (AxisLayoutProc) _ANSI_ARGS_((Graph *));
-typedef void (AxisDestroyProc) _ANSI_ARGS_((Graph *, GraphAxis *));
-
-struct GraphAxis {
-    AxisDisplayProc *displayProc;
-    AxisPrintProc *printProc;
-    AxisLayoutProc *layoutProc;
-    AxisDestroyProc *destroyProc;
-};
-
 /*
  * -------------------------------------------------------------------
  *
@@ -290,10 +328,10 @@ typedef void (CrosshairsDestroyProc) _ANSI_ARGS_((Graph *));
 
 struct GraphCrosshairs {
 
-    CrosshairsToggleProc *toggleProc; /* Toggle visiblity of crosshairs */
-    CrosshairsUpdateProc *updateProc; /* Update lengths of hairs */
-    CrosshairsConfigProc *configProc; /* Configure GC */
-    CrosshairsDestroyProc *destroyProc; /* Release X resources */
+    CrosshairsToggleProc *toggleProc;	/* Toggle visiblity of crosshairs */
+    CrosshairsUpdateProc *updateProc;	/* Update lengths of hairs */
+    CrosshairsConfigProc *configProc;	/* Configure GC */
+    CrosshairsDestroyProc *destroyProc;	/* Release X resources */
 
 
 };
@@ -321,8 +359,7 @@ struct Graph {
 				 * case the widget is destroyed and tkwin
 				 * become unavailable for querying */
     Tcl_Interp *interp;		/* Interpreter associated with graph widget */
-    GraphClassType type;	/* Type of graph: GRAPH_TYPE, BARCHART_TYPE,
-				 * or PIECHART_TYPE. */
+    GraphClassType type;	/* Type: either GRAPH or BARCHART */
     unsigned int flags;		/* Flags;  see below for definitions. */
     int reqWidth, reqHeight;	/* Requested size of graph window */
     int width, height;		/* Size of graph window or PostScript page */
@@ -330,21 +367,21 @@ struct Graph {
     int halo;			/* Maximum distance allowed between points
 				 * when searching for a point */
 
-    GraphPostScript *postscript;/* PostScript options */
-    GraphLegend *legendPtr;	/* Legend information */
-    GraphAxis *X, *Y;		/* X and Y Axis information */
-    GraphCrosshairs *crosshairs;
+    GraphPostScript *postscript;/* PostScript options: see bltGrPS.c */
+    GraphLegend *legendPtr;	/* Legend information: see bltGrLegd.c */
+    GraphAxis *axisArr[4];	/* Coordinate axis info: see bltGrAxis.c */
+    GraphCrosshairs *crosshairs;/* Crosshairs information: see bltGrHairs.c */
 
     char *scratchPtr;		/* Utility space for building strings. Points
 				 * to buffer of BUFSIZ bytes on the stack.
 				 * Currently used to create PostScript output
 				 * for the "postscript" command. */
 
-    int leftMargin;		/* The following are the requested sizes for */
-    int rightMargin;		/* the margins surrounding the plotting area */
-    int topMargin;		/* If non-zero, the requested margin is used */
-    int bottomMargin;		/* Otherwise the computed margin sizes are
-				 * used. */
+    int leftMargin;		/* Requested sizes for margins surrounding */
+    int rightMargin;		/* the plotting area. If non-zero, the */
+    int topMargin;		/* requested margin is used, otherwise the */
+    int bottomMargin;		/* computed margin sizes are used. */
+
     XPoint origin;		/* Origin of the graph */
     XPoint extreme;		/* Outer limit of graph */
 
@@ -365,8 +402,7 @@ struct Graph {
     double avgSymSize;		/* Average size of a symbol */
 
     char *title;		/* Graph title */
-    char *xTitle;		/* Axis titles */
-    char *yTitle;
+
     XFontStruct *fontPtr;	/* Font for title and axis labels */
     XColor *fgColorPtr;		/* Foreground color of title and axis labels */
     Cursor cursor;
@@ -410,73 +446,52 @@ struct Graph {
 #define LAYOUT_PENDING	        (1<<2)
 #define LAYOUT_ALL		(1<<3)
 
-/*
- * -------------------------------------------------------------------
- *
- * 	TextAttr --
- *
- * 	Represents a convenience structure to hold text attributes
- *	which determine how a text string is to be displayed on the
- *	window, or drawn with PostScript commands.  The alternative
- *	is to have drawing and printing routines with more parameters.
- * 	This seems like a more efficient and less cumbersome way.
- *
- * -------------------------------------------------------------------
- */
-typedef struct {
-    char *text;			/* Text string to be displayed  */
-    XFontStruct *fontPtr;	/* Font to use for string */
-    XColor *bgColorPtr;		/* Background color of string */
-    XColor *fgColorPtr;		/* Foreground color of string */
-    Tk_Anchor anchor;		/* Anchor type: used to position the string */
-    double theta;		/* Rotation of text in degrees. */
-    GC gc;			/* Graphics context to use when displaying the
-				 * string on the window */
-} TextAttr;
-
 /*
  * ---------------------- Forward declarations ------------------------
  */
 extern Pixmap Blt_CreateTextBitmap _ANSI_ARGS_((Display * display,
 	Drawable draw, XFontStruct *fontPtr, char *textStr, double theta,
-	unsigned int *bmWidthPtr, unsigned int *bmHeightPtr));
+	unsigned int *bmWPtr, unsigned int *bmHPtr));
 extern void Blt_DrawText _ANSI_ARGS_((Display * display, Drawable draw,
 	char *text, TextAttr *attrPtr, int x, int y));
 extern void Blt_EventuallyRedraw _ANSI_ARGS_((Graph *graphPtr));
-extern void Blt_GetBoundingBox _ANSI_ARGS_((int width, int height, double theta,
-	int *widthPtr, int *heightPtr, XPoint *pointArr));
+extern void Blt_GetBoundingBox _ANSI_ARGS_((unsigned int width,
+	unsigned int height, double theta, unsigned int *widthPtr,
+	unsigned int *heightPtr, XPoint *pointArr));
 extern Pixmap Blt_RotateBitmap _ANSI_ARGS_((Display * display, Drawable draw,
-	GC gc, Pixmap bitmap, int width, int height, double theta,
-	unsigned int *rotWidthPtr, unsigned int *rotHeightPtr));
+	GC gc, Pixmap bitmap, unsigned int width, unsigned int height,
+	double theta, unsigned int *rotWPtr, unsigned int *rotHPtr));
 extern void Blt_ComputeAxes _ANSI_ARGS_((Graph *graphPtr));
-extern XPoint Blt_WinPt _ANSI_ARGS_((Graph *graphPtr, double x, double y));
-extern int Blt_WinX _ANSI_ARGS_((Graph *graphPtr, double x));
-extern int Blt_WinY _ANSI_ARGS_((Graph *graphPtr, double y));
-extern int Blt_WinXAbs _ANSI_ARGS_((Graph *graphPtr, double x));
-extern int Blt_WinYAbs _ANSI_ARGS_((Graph *graphPtr, double y));
+
+extern int Blt_Transform _ANSI_ARGS_((GraphAxis *axis, double value));
+extern double Blt_InvTransform _ANSI_ARGS_((GraphAxis *axis, int coord));
+extern XPoint Blt_TransformPt _ANSI_ARGS_((Graph *graphPtr, double x, double y,
+	unsigned int axisFlags));
+extern int Blt_TransformDist _ANSI_ARGS_((GraphAxis *axis, double value));
+
 extern void Blt_StencilBitmap _ANSI_ARGS_((Display * display, Drawable draw,
 	GC gc, Pixmap bitmap, int x, int y, unsigned int width,
 	unsigned int height));
 extern unsigned int Blt_TextStringWidth _ANSI_ARGS_((XFontStruct *fontPtr,
 	char *text));
-extern XPoint Blt_TranslateBoxCoords _ANSI_ARGS_((int x, int y, int width,
-	int height, Tk_Anchor anchor));
+extern XPoint Blt_TranslateBoxCoords _ANSI_ARGS_((int x, int y,
+	unsigned int width, unsigned int height, Tk_Anchor anchor));
 extern XPoint Blt_TranslateTextCoords _ANSI_ARGS_((XFontStruct *fontPtr,
 	char *text, int x, int y, Tk_Anchor anchor));
-extern double Blt_WorldX _ANSI_ARGS_((Graph *graphPtr, int x));
-extern double Blt_WorldY _ANSI_ARGS_((Graph *graphPtr, int y));
 
-extern void Blt_PrintBitmap _ANSI_ARGS_((Graph *graphPtr,
-	Pixmap bitmap, int x, int y, int width, int height));
+extern void Blt_PrintLine _ANSI_ARGS_((Graph *graphPtr, XPoint *pointArr,
+	int numPoints));
+extern void Blt_PrintBitmap _ANSI_ARGS_((Graph *graphPtr, Pixmap bitmap, int x,
+	int y, unsigned int width, unsigned int height));
 extern void Blt_SetLineAttributes _ANSI_ARGS_((Graph *graphPtr,
 	XColor *colorPtr, int lineWidth, int lineDashes));
 extern void Blt_3DRectangleToPostScript _ANSI_ARGS_((Graph *graphPtr,
-	Tk_3DBorder border, int x, int y, int width, int height,
-	int borderWidth, int relief));
+	Tk_3DBorder border, int x, int y, unsigned int width,
+	unsigned int height, int borderWidth, int relief));
 extern void Blt_BackgroundToPostScript _ANSI_ARGS_((Graph *graphPtr,
 	XColor *colorPtr));
-extern void Blt_BitmapToPostScript _ANSI_ARGS_((Graph *graphPtr,
-	Pixmap bitmap, int x, int y, int width, int height, double theta,
+extern void Blt_BitmapToPostScript _ANSI_ARGS_((Graph *graphPtr, Pixmap bitmap,
+	int x, int y, unsigned int width, unsigned int height, double theta,
 	XColor *bgColorPtr));
 extern void Blt_FontToPostScript _ANSI_ARGS_((Graph *graphPtr,
 	XFontStruct *fontPtr));
@@ -491,8 +506,8 @@ extern void Blt_LinesToPostScript _ANSI_ARGS_((Graph *graphPtr,
 extern void Blt_PolygonToPostScript _ANSI_ARGS_((Graph *graphPtr,
 	XPoint *pointArr, int numPoints));
 extern void Blt_Print3DRectangle _ANSI_ARGS_((Graph *graphPtr,
-	Tk_3DBorder border, int x, int y, int width, int height,
-	int borderWidth, int relief));
+	Tk_3DBorder border, int x, int y, unsigned int width,
+	unsigned int height, int borderWidth, int relief));
 extern void Blt_RectangleToPostScript _ANSI_ARGS_((Graph *graphPtr,
 	int x, int y, unsigned int width, unsigned int height));
 extern void Blt_RectanglesToPostScript _ANSI_ARGS_((Graph *graphPtr,
